#include <SimpleModbusRtu_mega2560.h>     // comfileHMI와 통신하기위해 include합니다.
#include <Wire.h>                         // csg를 사용하기위해 include합니다.

// 상수값을 정의합니다.
#define     comm_baudRate   57600         // comfileHMI와의 통신속도를 정의합니다.
#define     comm_slave      1             // FA-DUINO가 사용될 슬레이브 주소값입니다.
#define     warning_light   38            // 경고등 번호를 나타냅니다.
#define     CSG_CURRENT     0             // 현재 측정된 ADC값이 뿌려질 CSG 주소입니다.
#define     CSG_SETTING     1             // 경고값이 뿌려질 CSG 주소입니다.

// CSG에서는 일반적인 FND처럼 각각의 LED를 알맞게 제어해야 숫자를 띄울 수 있습니다.
unsigned char     csg_lookupTable[10] = {0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x27,0x7f,0x6f};

// 모드버스 통신에 사용되는 변수입니다.
unsigned short    modbusRegister[10];     // 레지스터 영역을 담당합니다.
unsigned char     modbusCoil[10];         // 코일 영역을 담당합니다.

// 임의적으로 정의한 함수의 프로토타입입니다.
void csg_init(unsigned char channel);                           // CSG를 사용하기 앞서 초기화하는 함수입니다.
void csg_print(unsigned char channel,unsigned short value);     // CSG에 값을 뿌리기위해 사용합니다.

void setup() {
  pinMode(warning_light,OUTPUT);      // 경고등을 장착할 포트를 설정합니다.
  digitalWrite(warning_light,LOW);    // 초기상태는 소등되어있는 상태입니다.

  // CSG를 설정합니다.
  Wire.begin();             // CSG는 I2C 통신을 사용합니다.
  Wire.setClock(50000);     // CSG와의 통신을 빠르게 진행하기위해 50KHz로 설정합니다.
  csg_init(CSG_CURRENT);    // 측정된 ADC값이 출력되는 CSG를 초기화합니다.
  csg_init(CSG_SETTING);    // 설정한 경고값이 출력되는 CSG를 초기화합니다.

  // comfileHMI와의 통신을 설정합니다.
  usart1init(comm_baudRate);                                  // comfileHMI와는 RS232 채널 1에서 진행됩니다. _ 통신속도 : 57600 보레이트
  startSimpleModbus(comm_slave,modbusRegister,modbusCoil);    // 모드버스 프로토콜을 사용하기위해 관련 내용을 설정해줍니다.
}

void loop() {
  
  modbusRegister[0] = analogRead(4);  // ADC값을 읽어옵니다.

  // 경고값과 측정값을 비교하여 경고의 발생여부를 판단합니다.
  if(modbusRegister[1] <= modbusRegister[0])  modbusRegister[2] = 1;    // 경고발생!!!
  else                                        modbusRegister[2] = 0;    // 경고해제!!!

  // 경고등을 제어합니다. ___ 제어신호는 comfileHMI에서 관리합니다.
  digitalWrite(warning_light,modbusCoil[0]);

  // 측정한 ADC와 설정한 경고값을 CSG에 출력합니다.
  csg_print(CSG_CURRENT,modbusRegister[0]);
  csg_print(CSG_SETTING,modbusRegister[1]);
  
  delay(100);   // 임의의 지연입니다.
}

/* 임의로 선언한 함수의 정의부입니다. */

void csg_init(unsigned char channel)
{
  Wire.beginTransmission(0x38+channel);       // CSG의 주소표현 : 0x38 + 접근주소
  Wire.write(0x00);   // 제어 레지스터 선택.
  Wire.write(0x47);   // 컨트롤 플레그 설정. [ 12mA출력 | 깜빡임 사용안함 | 모든자리 사용(다이나믹) ]
  Wire.endTransmission();
}
void csg_print(unsigned char channel,unsigned short value)
{
  Wire.beginTransmission(0x38+channel);       // CSG의 주소표현 : 0x38 + 접근주소
  Wire.write(0x01); // 1번째 자리부터 연속출력.
  Wire.write(csg_lookupTable[(value/1000)%10]);   // 1번째 자리 ( 1000의 자리 )
  Wire.write(csg_lookupTable[(value/100)%10]);    // 2번째 자리 ( 100의 자리)
  Wire.write(csg_lookupTable[(value/10)%10]);     // 3번째 자리 ( 10의 자리)
  Wire.write(csg_lookupTable[value%10]);          // 4번째 자리 ( 1의 자리)
  Wire.endTransmission();
}

